@@ -1,32 +1,45 @@
 """This defines a basic wsgi application that provides RESTful routing."""
 
+from itertools import chain
+
 from werkzeug.exceptions import HTTPException
 from werkzeug.exceptions import MethodNotAllowed
 from werkzeug.exceptions import InternalServerError
-from werkzeug.wrappers import Request, BaseRequest, BaseResponse
+from werkzeug.routing import Map
+from werkzeug.wrappers import BaseRequest
+from werkzeug.wrappers import BaseResponse
+from werkzeug.wrappers import Request
 
 
 class RestApplication(object):
-    """Base REST application.
+    """Base REST application."""
 
-    This object uses a werkzeug url map to route a request to the appropriate
-    endpoint. Once an enpoint is found it attempts to call a method with the
-    same name as the requested verb. For example, a GET request will look for
-    the GET method of the endpoint to call.
+    def __init__(self, services=None, request_hooks=None, response_hooks=None):
 
-    All verbs are upper-cased.
+        self.services = services or []
 
-    Note::
+        # Build a list of url endpoints from the service interfaces.
+        url_map = []
+        converters = {}
+        for service in services:
+            url_map += (service.urls
+                        if hasattr(service, 'urls')
+                        else service['urls'])
 
-        This application requires that it be wrapped in the `RequestMiddleware`
-        found in the `wsgi` module.
-    """
+            # Check for custom converters used by an application.
+            custom_converters = {}
+            if hasattr(service, 'converters'):
+                custom_converters = service.converters
+            elif isinstance(service, dict) and 'converters' in service:
+                custom_converters = service['converters']
 
-    def __init__(self, url_map, pre_hooks=None, post_hooks=None):
+            converters = dict(chain(converters.iteritems(),
+                                    custom_converters.iteritems()))
 
-        self.url_map = url_map
-        self.pre_hooks = pre_hooks or []
-        self.post_hooks = post_hooks or []
+        self.url_map = Map(url_map, converters=converters)
+
+        self.request_hooks = request_hooks or []
+        self.response_hooks = response_hooks or []
 
     def __call__(self, environ, start_response):
 
@@ -123,7 +136,7 @@ class RestApplication(object):
 
     def process_request_hooks(self, request):
 
-        for hook in self.pre_hooks:
+        for hook in self.request_hooks:
 
             result = hook(request)
 
@@ -135,7 +148,7 @@ class RestApplication(object):
 
     def process_response_hooks(self, request, response):
 
-        for hook in self.post_hooks:
+        for hook in self.response_hooks:
 
             result = hook(request, response)
 
